<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0" xmlns:fc="urn:ibm:names:ITFIM:1.0:Federations"
	xmlns:fim="urn:ibm:names:ITFIM:1.0">

	<xsl:strip-space elements="*" />
	<xsl:output method="xml" version="1.0" encoding="utf-8"
		indent="yes" />

	<!--
		Initially we start with a copy of the document.
	-->
	<xsl:template match="@* | node()">
		<xsl:copy>
			<xsl:apply-templates select="@* | node()" />
		</xsl:copy>
	</xsl:template>

	<!--
		For each SAML "self" configuration (both IP and SP configurations), perform these transformations where the values exist:
		
		SAML.11						   -> SAML1.Version ("1.0" if false, "1.1" if true)
		SAML.SelfSuccinctId            -> SAML1.SourceID
		SAML.ArtifactCacheLifetime     -> SAML1.ArtifactLifeTime
		SAML.AllowIBMPROTOCOLExtension -> SAML1.AllowIBMProtocolExtension
		SAML.SelfKeyIdentifier         -> SAML1.SigningKeyIdentifier
		FED_PROT					   -> Convert to SAML1_1 if SAML.11 is true
		
		
		Add these parameters if they do not already exist:
		SAML1.Version - set to "1.0" if SAML.11 not present
		SAML1.SignArtifactResponse - set to "false" if not present (for IP only)
		SAML1.SignArtifactRequest - set to "false" if not present (for SP only)
		SAML1.SigningKeyIdentifier - set to "DefaultKeyStore_testkey" if not present AND SAML.SelfKeyIdentifier and not present
		
	-->

	<!--  find old SAML self's by finding a Self which has a FED_PROT entry with a value of SAML1_0 -->
	<xsl:template
		match="fc:Self[fc:EntityProperty[@name='FED_PROT']/fim:Value[text()='SAML1_0']]">
		<xsl:copy>
			<xsl:apply-templates select="@*" />
			<xsl:apply-templates>
				<xsl:with-param name="orig_saml11"
					select="fc:EntityProperty[@name='SAML.11']/fim:Value" />
				<xsl:with-param name="orig_selfkeyid"
					select="fc:EntityProperty[@name='SAML.SelfKeyIdentifier']/fim:Value" />
				<xsl:with-param name="new_selfkeyid"
					select="fc:EntityProperty[@name='SAML1.SigningKeyIdentifier']/fim:Value" />
				<xsl:with-param name="new_signartrequest"
					select="fc:EntityProperty[@name='SAML1.SignArtifactRequest']/fim:Value" />
				<xsl:with-param name="new_signartresponse"
					select="fc:EntityProperty[@name='SAML1.SignArtifactResponse']/fim:Value" />
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:template>

	<!--  Change SAML1_0 to SAML1_1 if they had a fixpack-version of SAML 1.1 
		This is also where we insert new parameters which apply to self configurations
	-->
	<xsl:template
		match="fc:EntityProperty[@name='FED_PROT'][fim:Value[text()='SAML1_0']]">
		<xsl:param name="orig_saml11" />
		<xsl:param name="orig_selfkeyid" />
		<xsl:param name="new_selfkeyid" />
		<xsl:param name="new_signartrequest" />
		<xsl:param name="new_signartresponse" />

		<fc:EntityProperty name="FED_PROT">
			<xsl:choose>
				<xsl:when test="$orig_saml11='true'">
					<fim:Value>SAML1_1</fim:Value>
				</xsl:when>
				<xsl:otherwise>
					<fim:Value>SAML1_0</fim:Value>
				</xsl:otherwise>
			</xsl:choose>
		</fc:EntityProperty>

		<!--  only insert a SAML version if there isn't a SAML.11 there which will be converted by another template -->
		<xsl:if test="not($orig_saml11)">
			<fc:EntityProperty name="SAML1.Version">
				<fim:Value>1.0</fim:Value>
			</fc:EntityProperty>
		</xsl:if>

		<!--  
				If neither the old nor new signing key exists, then this must be an old SP self config 
				otherwise it must be an IP self config (or an already processed SP config in which case we don't do
				anything anyway).
		-->
		<xsl:choose>
			<xsl:when
				test="not($orig_selfkeyid) and not($new_selfkeyid)">
				<fc:EntityProperty name="SAML1.SigningKeyIdentifier">
					<fim:Value>DefaultKeyStore_testkey</fim:Value>
				</fc:EntityProperty>

				<!--  insert SAML1.SignArtifactRequest if not there -->
				<xsl:if test="not($new_signartrequest)">
					<fc:EntityProperty name="SAML1.SignArtifactRequest">
						<fim:Value>false</fim:Value>
					</fc:EntityProperty>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!--  insert SAML1.SignArtifactResponse if not there (and not an already processed SP) -->
				<xsl:if test="not($new_signartresponse) and not($new_signartrequest)">
					<fc:EntityProperty name="SAML1.SignArtifactResponse">
						<fim:Value>false</fim:Value>
					</fc:EntityProperty>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<!--  Change the current boolean SAML.11 (if it exists) to SAML1.Version "1.0" or "1.1" -->
	<xsl:template match="fc:EntityProperty[@name='SAML.11']">
		<xsl:param name="orig_saml11" />
		<fc:EntityProperty name="SAML1.Version">

			<xsl:choose>
				<xsl:when test="$orig_saml11='true'">
					<fim:Value>1.1</fim:Value>
				</xsl:when>
				<xsl:otherwise>
					<fim:Value>1.0</fim:Value>
				</xsl:otherwise>
			</xsl:choose>

		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.SelfSuccinctID to SAML1.SourceID -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SelfSuccinctID']">
		<fc:EntityProperty name="SAML1.SourceID">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.ArtifactCacheLifetime to SAML1.ArtifactLifeTime -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.ArtifactCacheLifetime']">
		<fc:EntityProperty name="SAML1.ArtifactLifeTime">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.AllowIBMPROTOCOLExtension to SAML1.AllowIBMProtocolExtension -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.AllowIBMPROTOCOLExtension']">
		<fc:EntityProperty name="SAML1.AllowIBMProtocolExtension">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.SelfKeyIdentifier to SAML1.SigningKeyIdentifier -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SelfKeyIdentifier']">
		<fc:EntityProperty name="SAML1.SigningKeyIdentifier">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>


	<!--
		For each "partner" configuration (both IP and SP partner configurations), perform these transformations where the values exist:


			SAML.PartnerSuccinctID         -> SAML1.SourceID
			SAML.UseArtifactProfileForSSO  -> SAML1.UseArtifactProfileForSSO
			SAML.PartnerKeyIdentifier      -> SAML1.ValidateKeyIdentifier

		Add these parameters if they do not already exist:
			SAML1.ValidateKeyIdentifier - set to "DefaultKeyStore_testkey" if not present
			SAML1.ValidateArtifactRequest - set to "false" if not present (SP-partner only)
			SAML1.ValidateArtifactResponse - set to "false" if not present (IP-partner only)

		IP-Partners also have SOAP connection properties which need converting as follows:
			SAML.SOAPClientAuthentication - if false, set:
												SAML1.SOAPRequestClientCertAuth to false
												SAML1.SOAPRequestClientBasicAuth to false
                                            if true, set:
												SAML1.SOAPRequestClientCertAuth to true if SAML.SOAPClientAuthenticationType is "certificate" or missing, otherwise "false"
												SAML1.SOAPRequestClientBasicAuth to true if SAML.SOAPClientAuthenticationType is "basic" otherwise "false"
											if not present, do nothing since it's likely the transform has already been run.

			SAML.SOAPServerKeyIdentifier   -> SAML1.SOAPRequestServerCertAuthKeyIdentifier (or "DefaultTrustedKeyStore_testwebseal" if not present)
                                              also set SAML1.SOAPRequestServerCertAuth to true if SAML.SOAPServerKeyIdentifier present
			SAML.SOAPClientKeyIdentifier   -> SAML1.SOAPRequestClientCertAuthKeyIdentifier (or "DefaultKeyStore_testkey" if not present)
			SAML.SOAPBasicAuthenticationUserid -> SAML1.SOAPRequestClientBasicAuthUser (or "dummyuser" if not present)
			SAML.SOAPBasicAuthenticationPassword -> SAML1.SOAPRequestClientBasicAuthPassword (or "dummyuser_password" if not present)

			SAML.SOAPClientAuthenticationType -> remove
	-->

	<!--  find old SAML partners by finding a partner which has a SAML.PartnerSuccinctID entry -->
	<xsl:template
		match="fc:Partner[fc:EntityProperty[@name='SAML.PartnerSuccinctID']]">
		<xsl:copy>
			<xsl:apply-templates select="@*" />
			<xsl:apply-templates>
				<xsl:with-param name="orig_partnerkeyid"
					select="fc:EntityProperty[@name='SAML.PartnerKeyIdentifier']/fim:Value" />
				<xsl:with-param name="new_partnerkeyid"
					select="fc:EntityProperty[@name='SAML1.ValidateKeyIdentifier']/fim:Value" />
				<xsl:with-param name="old_useartforsso"
					select="fc:EntityProperty[@name='SAML.UseArtifactProfileForSSO']/fim:Value" />
				<xsl:with-param name="new_useartforsso"
					select="fc:EntityProperty[@name='SAML1.UseArtifactProfileForSSO']/fim:Value" />
				<xsl:with-param name="old_soapclientauth"
					select="fc:EntityProperty[@name='SAML.SOAPClientAuthentication']/fim:Value" />
				<xsl:with-param name="old_soapclientauthtype"
					select="fc:EntityProperty[@name='SAML.SOAPClientAuthenticationType']/fim:Value" />
				<xsl:with-param name="old_soapserverkeyid"
					select="fc:EntityProperty[@name='SAML.SOAPServerKeyIdentifier']/fim:Value" />
				<xsl:with-param name="old_soapclientkeyid"
					select="fc:EntityProperty[@name='SAML.SOAPClientKeyIdentifier']/fim:Value" />
				<xsl:with-param name="old_soapclientbauser"
					select="fc:EntityProperty[@name='SAML.SOAPBasicAuthenticationUserid']/fim:Value" />
				<xsl:with-param name="old_soapclientbapwd"
					select="fc:EntityProperty[@name='SAML.SOAPBasicAuthenticationPassword']/fim:Value" />
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:template>

	<!--  Change any current SAML.PartnerSuccinctID to SAML1.SourceID 
		  This is also where we add any general (IP or SP) params needed by the new configuration that
		  aren't already there.
	-->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.PartnerSuccinctID']">
		<xsl:param name="orig_partnerkeyid" />
		<xsl:param name="new_partnerkeyid" />
		<xsl:param name="old_useartforsso" />
		<xsl:param name="new_useartforsso" />
		<xsl:param name="old_soapserverkeyid" />
		<xsl:param name="old_soapclientkeyid" />
		<xsl:param name="old_soapclientbauser" />
		<xsl:param name="old_soapclientbapwd" />


		<fc:EntityProperty name="SAML1.SourceID">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>

		<xsl:if test="not($orig_partnerkeyid) and not($new_partnerkeyid)">
			<fc:EntityProperty name="SAML1.ValidateKeyIdentifier">
				<fim:Value>DefaultKeyStore_testkey</fim:Value>
			</fc:EntityProperty>
		</xsl:if>

		<!-- 
		 	 If this is IP-partner, add SAML1.ValidateArtifactResponse and add any of the missing SAML partner client authentication entries.
			 Otherwise this is SP-partner, add SAML1.ValidateArtifactRequest
		-->
		<xsl:choose>
			<xsl:when test="not($old_useartforsso) and not($new_useartforsso)">
				<fc:EntityProperty name="SAML1.ValidateArtifactResponse">
					<fim:Value>false</fim:Value>
				</fc:EntityProperty>

				<!--  now SOAP client-auth missing params -->
				<xsl:if test="not($old_soapserverkeyid)">
					<fc:EntityProperty name="SAML1.SOAPRequestServerCertAuthKeyIdentifier">
						<fim:Value>DefaultTrustedKeyStore_testwebseal</fim:Value>
					</fc:EntityProperty>
				</xsl:if>

				<xsl:if test="not($old_soapclientkeyid)">
					<fc:EntityProperty name="SAML1.SOAPRequestClientCertAuthKeyIdentifier">
						<fim:Value>DefaultKeyStore_testkey</fim:Value>
					</fc:EntityProperty>
				</xsl:if>

				<xsl:if test="not($old_soapclientbauser)">
					<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuthUser">
						<fim:Value>dummyuser</fim:Value>
					</fc:EntityProperty>
				</xsl:if>

				<xsl:if test="not($old_soapclientbapwd)">
					<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuthPassword">
						<fim:Value>dummyuser_password</fim:Value>
					</fc:EntityProperty>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<fc:EntityProperty name="SAML1.ValidateArtifactRequest">
					<fim:Value>false</fim:Value>
				</fc:EntityProperty>
			</xsl:otherwise>
		</xsl:choose>
		

	</xsl:template>

	<!--  Change any current SAML.UseArtifactProfileForSSO to SAML1.UseArtifactProfileForSSO -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.UseArtifactProfileForSSO']">
		<fc:EntityProperty name="SAML1.UseArtifactProfileForSSO">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.PartnerKeyIdentifier to SAML1.ValidateKeyIdentifier -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.PartnerKeyIdentifier']">
		<fc:EntityProperty name="SAML1.ValidateKeyIdentifier">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPClientAuthentication']">
		<xsl:param name="old_soapclientauth" />
		<xsl:param name="old_soapclientauthtype" />
		<xsl:choose>
			<xsl:when test="$old_soapclientauth = 'true'">
				<xsl:choose>
					<xsl:when test="not($old_soapclientauthtype) or $old_soapclientauthtype='certificate'">
						<fc:EntityProperty name="SAML1.SOAPRequestClientCertAuth">
							<fim:Value>true</fim:Value>
						</fc:EntityProperty>
					</xsl:when>
					<xsl:otherwise>
						<fc:EntityProperty name="SAML1.SOAPRequestClientCertAuth">
							<fim:Value>false</fim:Value>
						</fc:EntityProperty>
					</xsl:otherwise>
				</xsl:choose>
			
				<xsl:choose>
					<xsl:when test="$old_soapclientauthtype='basic'">
						<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuth">
							<fim:Value>true</fim:Value>
						</fc:EntityProperty>
					</xsl:when>
					<xsl:otherwise>
						<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuth">
							<fim:Value>false</fim:Value>
						</fc:EntityProperty>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<fc:EntityProperty name="SAML1.SOAPRequestClientCertAuth">
					<fim:Value>false</fim:Value>
				</fc:EntityProperty>
				<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuth">
					<fim:Value>false</fim:Value>
				</fc:EntityProperty>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<!--  Change any current SAML.SOAPServerKeyIdentifier to SAML1.SOAPRequestServerCertAuthKeyIdentifier
		  Also add SAML1.SOAPRequestServerCertAuth to true in this case
    -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPServerKeyIdentifier']">
		<fc:EntityProperty name="SAML1.SOAPRequestServerCertAuthKeyIdentifier">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
		<fc:EntityProperty name="SAML1.SOAPRequestServerCertAuth">
			<fim:Value>true</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.SOAPClientKeyIdentifier to SAML1.SOAPRequestClientCertAuthKeyIdentifier -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPClientKeyIdentifier']">
		<fc:EntityProperty name="SAML1.SOAPRequestClientCertAuthKeyIdentifier">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>


	<!--  Change any current SAML.SOAPBasicAuthenticationUserid to SAML1.SOAPRequestClientBasicAuthUser -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPBasicAuthenticationUserid']">
		<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuthUser">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Change any current SAML.SOAPBasicAuthenticationPassword to SAML1.SOAPRequestClientBasicAuthPassword -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPBasicAuthenticationPassword']">
		<fc:EntityProperty name="SAML1.SOAPRequestClientBasicAuthPassword">
			<fim:Value>
				<xsl:value-of select="fim:Value" />
			</fim:Value>
		</fc:EntityProperty>
	</xsl:template>

	<!--  Remove SAML.SOAPClientAuthenticationType -->
	<xsl:template
		match="fc:EntityProperty[@name='SAML.SOAPClientAuthenticationType']">
	</xsl:template>




	<!--  
		Finally, modify the entity endpoint values:

		For SAML indentity providers:
			Change SAML.SignonEndpoint to SAML1.IntersiteTransferService
			Change SAML.IPAssertionEndpoint to SAML1.ArtifactResolutionService

		For SAML service providers:
			Change SAML.SignonEndpoint to SAML1.AssertionConsumerService

	-->
	<xsl:template
		match="fc:Entity[@role='ip']/fc:ProtocolEndpoints/fc:ProtocolEndpoint[@type='SAML.IPAssertionEndpoint']" >
				<fc:ProtocolEndpoint
					endpoint="{@endpoint}"
					type="SAML1.ArtifactResolutionService" />
	</xsl:template>

	<xsl:template
		match="fc:Entity[@role='ip']/fc:ProtocolEndpoints/fc:ProtocolEndpoint[@type='SAML.SignonEndpoint']" >
				<fc:ProtocolEndpoint
					endpoint="{@endpoint}"
					type="SAML1.IntersiteTransferService" />
	</xsl:template>

	<xsl:template
		match="fc:Entity[@role='sp']/fc:ProtocolEndpoints/fc:ProtocolEndpoint[@type='SAML.SignonEndpoint']" >
				<fc:ProtocolEndpoint
					endpoint="{@endpoint}"
					type="SAML1.AssertionConsumerService" />
	</xsl:template>

</xsl:stylesheet>
